using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using TrainingSvc.Data;
using TrainingSvc.DTOs;
using TrainingSvc.Entities;

namespace TrainingSvc.Controllers;

[ApiController]
[Route("api/training/[controller]")]
public class ExerciceController : ControllerBase
{
    private readonly TrainingDbContext _context;
    private readonly IMapper _mapper;

    public ExerciceController(TrainingDbContext context, IMapper mapper)
    {
        _context = context;
        _mapper = mapper;
    }

    [HttpGet]
    public async Task<ActionResult<IEnumerable<ExerciceDto>>> GetAll()
    {
        var list = await _context.ExerciceInstances
            .Include(e => e.ExerciceTemplate)
            .ToListAsync();
        return Ok(_mapper.Map<IEnumerable<ExerciceDto>>(list));
    }

    [HttpGet("{id}")]
    public async Task<ActionResult<ExerciceDto>> GetById(string id)
    {
        var entity = await _context.ExerciceInstances
            .Include(e => e.ExerciceTemplate)
            .FirstOrDefaultAsync(e => e.Id == id);
        if (entity == null) return NotFound();
        return _mapper.Map<ExerciceDto>(entity);
    }

    [HttpPost]
    public async Task<ActionResult<ExerciceDto>> Create([FromBody] CreateExerciceInstanceDto dto)
    {
        var entity = _mapper.Map<ExerciceInstance>(dto);
        _context.ExerciceInstances.Add(entity);
        await _context.SaveChangesAsync();
        await _context.Entry(entity).Reference(e => e.ExerciceTemplate).LoadAsync();
        return CreatedAtAction(nameof(GetById), new { id = entity.Id }, _mapper.Map<ExerciceDto>(entity));
    }

    [HttpPut("{id}")]
    public async Task<IActionResult> Update(string id, [FromBody] UpdateExerciceInstanceDto dto)
    {
        var entity = await _context.ExerciceInstances
            .Include(e => e.ExerciceTemplate)
            .FirstOrDefaultAsync(e => e.Id == id);
        if (entity == null) return NotFound();
        _mapper.Map(dto, entity);
        await _context.SaveChangesAsync();
        return NoContent();
    }

    [HttpDelete("{id}")]
    public async Task<IActionResult> Delete(string id)
    {
        var entity = await _context.ExerciceInstances.FindAsync(id);
        if (entity == null) return NotFound();
        _context.ExerciceInstances.Remove(entity);
        await _context.SaveChangesAsync();
        return NoContent();
    }
}