using CatalogService.Entities;
using Microsoft.EntityFrameworkCore;
using TrainingSvc.Entities;

namespace TrainingSvc.Data;

public class TrainingDbInitializer
{
    public static void InitDb(WebApplication app)
    {
        using var scope = app.Services.CreateScope();
        SeedData(scope.ServiceProvider.GetService<TrainingDbContext>());
    }

    private static void SeedData(TrainingDbContext context)
    {
        context.Database.Migrate();

        // 1. Seed ExerciceTemplates si vide
        if (!context.ExerciceTemplates.Any())
        {
            var exercices = new List<ExerciceTemplate>()
            {
                new ExerciceTemplate
                {
                    Id = Guid.NewGuid().ToString(),
                    Name = "Squat",
                    Description = "Squat is a compound exercise that targets the lower body, primarily the quadriceps, hamstrings, and glutes.",
                    Target = Shared.Enum.ETarget.Legs,
                    ImageUrl = "images/squat.jpg",
                    VideoUrl = "https://www.youtube.com/watch?v=dQw4w9WgXcQ&ab_channel=RickAstley",
                },
                new ExerciceTemplate
                {
                    Id = Guid.NewGuid().ToString(),
                    Name = "Bench Press",
                    Description = "Bench Press is a compound exercise that primarily targets the chest, shoulders, and triceps.",
                    Target = Shared.Enum.ETarget.Chest,
                    ImageUrl = "images/bench_press.jpg",
                    VideoUrl = "https://www.youtube.com/watch?v=dQw4w9WgXcQ&ab_channel=RickAstley",
                },
                new ExerciceTemplate
                {
                    Id = Guid.NewGuid().ToString(),
                    Name = "Deadlift",
                    Description = "Deadlift is a compound exercise that primarily targets the back, glutes, and hamstrings.",
                    Target = Shared.Enum.ETarget.Back,
                    ImageUrl = "images/deadlift.jpg",
                    VideoUrl = "https://www.youtube.com/watch?v=dQw4w9WgXcQ&ab_channel=RickAstley",
                },
                new ExerciceTemplate
                {
                    Id = Guid.NewGuid().ToString(),
                    Name = "Shoulder Press",
                    Description = "Shoulder Press is a compound exercise that primarily targets the shoulders and triceps.",
                    Target = Shared.Enum.ETarget.Arms,
                    ImageUrl = "images/shoulder_press.jpg",
                    VideoUrl = "https://www.youtube.com/watch?v=dQw4w9WgXcQ&ab_channel=RickAstley",
                },
                new ExerciceTemplate
                {
                    Id = Guid.NewGuid().ToString(),
                    Name = "Running on Treadmill",
                    Description = "Running on Treadmill is a cardiovascular exercise that primarily targets the legs and improves overall fitness.",
                    Target = Shared.Enum.ETarget.Cardio,
                    ImageUrl = "images/running_treadmill.jpg",
                    VideoUrl = "https://www.youtube.com/watch?v=dQw4w9WgXcQ&ab_channel=RickAstley",
                },
            };
            context.AddRange(exercices);
            context.SaveChanges();
        }

        // 2. Créer un TrainingProgram si aucun n'existe
        var program = context.TrainingPrograms.FirstOrDefault();
        if (program == null)
        {
            program = new TrainingProgram
            {
                Id = Guid.NewGuid().ToString(),
                Name = "Programme de test"
            };
            context.TrainingPrograms.Add(program);
            context.SaveChanges();
        }

        // 3. Créer une session de test liée au programme si aucune n'existe
        var session = context.Sessions.FirstOrDefault();
        if (session == null)
        {
            session = new Session
            {
                Id = Guid.NewGuid().ToString(),
                Name = "Session de test",
                TrainingProgramId = program.Id
            };
            context.Sessions.Add(session);
            context.SaveChanges();
        }

        // 4. Seed ExerciceInstances si vide
        if (!context.ExerciceInstances.Any() && context.ExerciceTemplates.Any())
        {
            var templates = context.ExerciceTemplates.ToList();

            var instances = new List<ExerciceInstance>
            {
                new ExerciceInstance
                {
                    Name = "Squat Série 1",
                    ExerciceTemplateId = templates.First().Id,
                    Duration = 60,
                    NbSets = 4,
                    NbReps = 10,
                    RestingTime = 90,
                    Weight = 80,
                    IsDone = false,
                    SessionId = session.Id
                },
                new ExerciceInstance
                {
                    Name = "Bench Press Série 1",
                    ExerciceTemplateId = templates.Skip(1).First().Id,
                    Duration = 45,
                    NbSets = 3,
                    NbReps = 12,
                    RestingTime = 60,
                    Weight = 70,
                    IsDone = false,
                    SessionId = session.Id
                }
            };

            context.ExerciceInstances.AddRange(instances);
            context.SaveChanges();
        }
    }
}